/*
 * Messuhr-/Messschieber-Schnittstelle
 *
 * main.c
 *
 * Einlesen der Daten einer Messuhr im 24-Bit-Protokoll und Darstellung auf einem
 * virtuellen COM-Port per USB
 *
 * Hardware: -> siehe hardware_config.h
 *
 * Die Implementierung basiert groesstenteils auf einem Implementierungsbeispiel
 * der USB-FS-Library
 * STM32_USB-FS-Device_Lib_V4.1.0\Projects\Virtual_COM_Port
 *
 * $Date: 2019-02-09 12:08:16 +0100 (Sa, 09 Feb 2019) $
 * $Revision: 17 $
 *
 * 06.02.2019 Nicolas
 */


/* Includes ------------------------------------------------------------------*/
#include "hardware_config.h"
#include "hw_config.h"
#include "usb_lib.h"
#include "usb_desc.h"
#include "usb_pwr.h"
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include <float.h>



/* Extern variables ----------------------------------------------------------*/
extern __IO uint8_t Receive_Buffer[64];
extern __IO  uint32_t Receive_length ;
extern __IO  uint32_t length;

/* Private variables ---------------------------------------------------------*/
uint8_t Send_Buffer[64];
uint32_t packet_sent=1;
uint32_t packet_receive=1;


/* Private function prototypes -----------------------------------------------*/
void EXTI0_Config(void);




volatile uint32_t rawdata = 0;           // Data telegram from dial gauge/caliper
volatile bool newDataAvailable = false;  // Set in ISR, cleared in main()
volatile uint32_t gTime_ms = 0;          // Global time stamp, set only in SysTick handle


// Daten-Telegramme 24 Bit
// Raw Data |inch | n.a.| n.a.|sign| MSB | ..... | LSB |
#define DATA_IS_INCH     0x80000000
#define DATA_IS_NEGATIVE 0x10000000

#define DATA_TIMEOUT_MS 50



/* Interprete data telegrams (24 Bits) and send to virtual COM Port as text string */
int main(void)
{
    SystemInit();
    SysTick_Config(SystemCoreClock/1000*(SYSTICK_TIMER_INTERVAL_MS));

    Set_System();
    Set_USBClock();
    USB_Interrupts_Config();
    USB_Init();
    EXTI0_Config();

    // Input Data-Pin
    GPIO_InitTypeDef   GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Pin = INPUT_DAT_Pin;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(INPUT_DAT_GPIO, &GPIO_InitStructure);


    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, ENABLE);

    /* Configure the GPIO_LED pin */
    GPIO_InitStructure.GPIO_Pin = SPARE0_Pin;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

    GPIO_Init(SPARE0_GPIO, &GPIO_InitStructure);

    while (1)
    {
        if (bDeviceState == CONFIGURED)
        {
            /* Receiver virtual UART, just to discard it */
            CDC_Receive_DATA();
            //Receive_length = 0;

            if( newDataAvailable == true )
            {
                if( packet_sent == 1 )
                {
                    uint32_t data = rawdata;
                    uint32_t uvalue = (data>>8) & 0x000FFFFF;
                    int bufferlen;

                    float value = uvalue;

                    if( data & DATA_IS_NEGATIVE )
                    {
                        value = -value;
                    }

                    if( data & DATA_IS_INCH )
                    {
                        // LSB is 0.005 inch
                        value *= 0.0005f;
                        bufferlen = snprintf(Send_Buffer, 30, "0x%06lx | %7.3f %s\r\n",
                            data>>8, value, "inch");
                    }
                    else
                    {
                        // LSB is 0.01 mm
                        value *= 0.01f;
                        bufferlen = snprintf(Send_Buffer, 30, "0x%06lx | %6.2f %s\r\n",
                            data>>8, value, "mm");
                    }


                    CDC_Send_DATA (Send_Buffer, bufferlen);

                    newDataAvailable = false;
                }
            }
        }
    }
}



/* Timestamps */
void SysTick_Handler(void)
{
    gTime_ms++;
}



/* On rising edge of clock signal, read data bit to 24 bit shift register
 * Not fully received data words will be discarded after timeout */
void EXTI0_IRQHandler(void)
{
    static uint32_t timeLast = 0;
    static uint32_t shiftRegister = 0;
    static uint32_t nBit = 0;

    if(EXTI_GetITStatus(EXTI_Line0) != RESET)
    {
        if( (gTime_ms - timeLast) > DATA_TIMEOUT_MS )
        {
            // Timeout
            SPARE0_GPIO->ODR ^= SPARE0_Pin;
            shiftRegister = 0;
            nBit = 0;
        }
        timeLast = gTime_ms;


        // Read input to shift Register
        // Fill Shift Register from right to left (MSB first)
        shiftRegister >>= 1;
        if( INPUT_DAT_GPIO->IDR & INPUT_DAT_Pin )
        {
            shiftRegister |= 0x80000000;
        }
        nBit++;

        // Pass full word to other functions
        if( nBit == 24 )
        {
            rawdata = shiftRegister;
            newDataAvailable = true;

            // cleanup of shiftRegister will be done by timeout
        }

        /* Clear the  EXTI line 0 pending bit */
        EXTI_ClearITPendingBit(EXTI_Line0);
    }
}




/* Configure EXT0 as Pin Change Interrupt (rising edge only) */
void EXTI0_Config(void)
{
    EXTI_InitTypeDef   EXTI_InitStructure;
    GPIO_InitTypeDef   GPIO_InitStructure;
    NVIC_InitTypeDef   NVIC_InitStructure;

    /* Enable GPIOx clock */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

    /* Configure PA.00 pin as input  */
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(INPUT_CLK_GPIO, &GPIO_InitStructure);

    /* Enable AFIO clock */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);

    /* Connect EXTI0 Line to PA.00 pin */
     GPIO_EXTILineConfig(GPIO_PortSourceGPIOA, GPIO_PinSource0);

    /* Configure EXTI0 line */
    EXTI_InitStructure.EXTI_Line = EXTI_Line0;

    EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
    EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
    EXTI_InitStructure.EXTI_LineCmd = ENABLE;
    EXTI_Init(&EXTI_InitStructure);

    /* Enable and set EXTI0 Interrupt to the lowest priority */
    NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQn;
    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x0F;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0x0F;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
}



